import logging
import logging.handlers
import colorlog
import os
import uuid
from ez_logger.file_logger import FileLogger
from ez_logger.console_logger import ConsoleLogger


class TenantLogger(FileLogger):
    """
    TenantLogger is a file logger that can be used when the application uses a single log file for processing multiple clients.
    It descends from FileFormatter and overrides the formatter, log, warn, debug, critical methods

    Parameters
    ----------
    name : `string`, `optional`
         Name of the logger.
    log_level : `string`, `optional`
         log_level
         Default value is info. Other allowed values are debug, warning, error or critical
    console_logging : `bool`, `optional`
         Should log messages be shown on the console?
         Default is True
    log_file : `string`, `optional`
         If not specified, then it will use a file_name { name }.log. If name is none then the log file will be app.log
    log_dir : `string`, `optional`
         The directory where the log_file is created. If not specified, then log_file will be created in the current directory.
         If directory does not exist, or the user does not have write permission, then log_file will be created in current directory.
    Examples
    --------
    >>> logger = TenantLogger(log_file='multi-tenant.log', log_dir='/tmp')
    >>> logger.info(message='Process started', tenant='tenant1')
    """

    def __init__(self, console_logging=True, log_level='info', name=None, log_file=None,
                 log_dir=None):
        super(TenantLogger, self).__init__(
            name=name,
            log_level=log_level,
            console_logging=console_logging,
            log_file=log_file,
            log_dir=log_dir
        )

    def _get_file_formatter(self):
        return '%(log_color)s%(asctime)s - %(name)s - %(tenant)s - %(levelname)s - %(message)s'

    def info(self, message, tenant=None):
        """
        Writes info messages, these messages will only be written if
        log_level is set to debug or info.
        Formatter will use the tenant name in the log record.
        """
        self.logger.info(message, extra={'tenant': tenant})

    def error(self, message, tenant=None):
        """
        Writes error messages, these messages will only be written if
        log_level is set to debug, info, warning or error.
        Formatter will use the tenant name in the log record.
        """
        self.logger.error(message, extra={'tenant': tenant})

    def warning(self, message, tenant=None):
        """
        Writes warning messages, these messages will only be written
        if log_level is set to debug, info or warning.
        Formatter will use the tenant name in the log record.
        """
        self.logger.warning(message, extra={'tenant': tenant})

    def critical(self, message, tenant=None):
        """
        Writes error messages, these messages will be written
        regardless of the log_level (it could be set to debug,
        info, warning, error or critical).
        Formatter will use the tenant name in the log record.
        """
        self.logger.critical(message, extra={'tenant': tenant})

    def debug(self, message, tenant=None):
        """
        Writes debug messages, these messages will only be written
        if log_level is set to debug.
        Formatter will use the tenant name in the log record.
        """
        self.logger.debug(message, extra={'tenant': tenant})



import logging
import colorlog


class BaseLogger(object):
    """
    BaseLogger is the parent class, that encapsulates the common functionality
    for different kinds of loggers.

    ConsoleLogger and FileLogger descend from BaseLogger

    Don't create BaseLogger objects. It is an abstract class. Use ConsoleLogger
    or FileLogger or TenantLogger.
    """

    def __init__(self, name, log_level):
        self.name = name
        self.log_level = log_level

    def _get_logger_name(self):
        if self.name is None:
            return 'app'
        else:
            return self.name

    def _get_log_level(self):
        log_level_dict = {
            'info': logging.INFO,
            'debug': logging.DEBUG,
            'warning': logging.WARNING,
            'error': logging.ERROR,
            'critical': logging.CRITICAL
        }
        if self.log_level not in log_level_dict.keys():
            # wrong log_level specified, so defaulting to info.
            return logging.INFO
        else:
            return log_level_dict[self.log_level]

    def _get_console_handler(self):
        console_handler = logging.StreamHandler()
        console_formatter = colorlog.ColoredFormatter(
            '%(log_color)s%(message)s'
        )
        console_handler.setFormatter(console_formatter)
        return console_handler

    def __getLogger(self):
        """
        Returns the logger object. This method needs to be implemented
        by the child class
        """
        raise NotImplementedError

    def info(self, message):
        """
        Writes info messages, these messages will only be written if
        log_level is set to debug or info.
        """
        self.logger.info(message)

    def error(self, message):
        """
        Writes error messages, these messages will only be written if
        log_level is set to debug, info, warning or error.
        """
        self.logger.error(message)

    def warning(self, message):
        """
        Writes warning messages, these messages will only be written
        if log_level is set to debug, info or warning.
        """
        self.logger.warning(message)

    def critical(self, message):
        """
        Writes error messages, these messages will be written
        regardless of the log_level (it could be set to debug,
        info, warning, error or critical).
        """
        self.logger.critical(message)

    def debug(self, message):
        """
        Writes debug messages, these messages will only be written
        if log_level is set to debug.
        """
        self.logger.debug(message)



import logging
from ez_logger.base_logger import BaseLogger


class ConsoleLogger(BaseLogger):
    """
    ConsoleLogger provides a logger that could be used for simple log messages to console.
    It uses colorlog formatter to color code log messages for different log levels.

    Parameters
    ----------
    name : `string`, `optional`
         Name of the logger.
    log_level : `string`, `optional`
         log_level
         Default value is info. Other allowed values are debug, warning, error or critical

    Examples
    --------
    >>> logger = ConsoleLogger()
    >>> logger.info(message='process started')
    """

    def __init__(self, name=None, log_level='info'):
        super(ConsoleLogger, self).__init__(
            name=name,
            log_level=log_level
        )
        self.logger = self.__getLogger()

    def __getLogger(self):
        # Create a logger object
        logger = logging.getLogger(
            name=self._get_logger_name()
        )

        # Set the log level
        logger.setLevel(self._get_log_level())

        # Add console handler
        logger.addHandler(self._get_console_handler())
        return logger



from setuptools import setup, find_packages
from codecs import open
from os import path

__version__ = '1.1.10'

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# get the dependencies and installs

all_reqs = [
    'colorlog==4.0.2'
]
install_requires = [x.strip() for x in all_reqs if 'git+' not in x]
dependency_links = [x.strip().replace('git+', '') for x in all_reqs if x.startswith('git+')]

setup(
    name='ez-logger',
    version=__version__,
    description='ez-logger allows you to a logger for everyday operations',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='BSD',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython'
    ],
    keywords='',
    packages=find_packages(exclude=['docs', 'tests*', '.idea']),
    include_package_data=True,
    author='Rakesh V',
    install_requires=install_requires,
    dependency_links=dependency_links,
    author_email='varma.rakesh@gmail.com',
    url='https://github.com/varmarakesh/ez-logger'
)



import logging
import logging.handlers
import colorlog
import os
import uuid
from ez_logger.base_logger import BaseLogger
from ez_logger.console_logger import ConsoleLogger


class FileLogger(BaseLogger):
    """
    FileLogger provides a logger that could be used for batch processing applications.
    It uses colorlog formatter to color code log messages for different log levels.

    Parameters
    ----------
    name : `string`, `optional`
         Name of the logger.
    log_level : `string`, `optional`
         log_level
         Default value is info. Other allowed values are debug, warning, error or critical
    console_logging : `bool`, `optional`
         Should log messages be shown on the console?
         Default is True
    log_file : `string`, `optional`
         If not specified, then it will use a file_name { name }.log. If name is none then the log file will be app.log
    log_dir : `string`, `optional`
         The directory where the log_file is created. If not specified, then log_file will be created in the current directory.
         If directory does not exist, or the user does not have write permission, then log_file will be created in current directory.
    Examples
    --------
    >>> logger = FileLogger(log_file='test.log', log_dir='/tmp')
    >>> logger.info(message='process started')
    """

    def __init__(self, console_logging=True, log_level='info', name=None, log_file=None,
                 log_dir=None):
        super(FileLogger, self).__init__(
            name=name,
            log_level=log_level
        )
        self.console_logging = console_logging
        self.log_file = log_file
        self.log_dir = log_dir

        # set the logger
        self.logger = self.__getLogger()

    def __check_dir(self, dir_path):
        dummypath = os.path.join(dir_path, str(uuid.uuid4()))
        try:
            with open(dummypath, 'w'):
                pass
            os.remove(dummypath)
            return True
        except IOError:
            return False

    def __getLogfile(self):
        # set log_file
        if self.log_file is None:
            if self.name is None:
                self.log_file = 'app.log'
            else:
                self.log_file = '{0}.log'.format(self.name)

        if not self.log_dir:
            logfile = self.log_file
        else:
            # checking if the self.log_dir exists or has write access.
            if self.__check_dir(self.log_dir):
                logfile = '{0}/{1}'.format(self.log_dir, self.log_file)
            else:
                logfile = self.log_file
                # if self.log_dir does not exist or has write access.
                # then display a console warning.
                console_logger = ConsoleLogger()
                console_logger.warning(
                    'Log directory {0} does not exist or user does not have write permission. '
                    'So creating the log file in the local directory.'.format(self.log_dir)
                )
        return logfile

    def _get_file_formatter(self):
        """
        _get_file_formatter indicates how the log file record should be formatted
        It can be overridden by the child class
        :return:
        """
        return '%(log_color)s%(asctime)s - %(name)s - %(levelname)s - %(message)s'

    def __get_file_handler(self):
        """
        __get_file_handler creates the RotatingFileHandler.
        """
        file_handler = logging.handlers.RotatingFileHandler(
            self.__getLogfile(),
            maxBytes=5 * 1024 * 1024,
            backupCount=2
        )
        file_formatter = colorlog.ColoredFormatter(
            self._get_file_formatter()
        )
        file_handler.setFormatter(file_formatter)
        return file_handler

    def __getLogger(self):
        # Create a logger object
        logger = logging.getLogger(
            name=self._get_logger_name()
        )

        # Set the log level
        logger.setLevel(self._get_log_level())

        # Remove if any loggers already exist. Logger is a singleton, so even if separate logger
        # are created, the handlers get appended.
        while len(logger.handlers) > 0:
            logger.handlers.pop()

        # Add file handler
        logger.addHandler(self.__get_file_handler())

        # Add console handler, if needed
        if self.console_logging:
            logger.addHandler(self._get_console_handler())
        return logger



import logging


class DefaultLogger(object):
    """
    Logger class provides a default logger object. If for any reason the user wants to use a default logger,
    this allows the user to do so without having to import the logging module.
    """
    def __init__(self):
        self.logger = logging.getLogger(__name__)
